! This file is part of HiggsBounds
!  -KW
!************************************************************	
subroutine initialize_HiggsBounds(nHiggs,whichexpt_in)
! This the first Higgsbounds subroutine that should be called
! by the user.
! It calls subroutines to read in the tables of Standard Model data,
! read in the tables of LEP and Tevatron data,
! set up lists of processes which should be checked against 
! the experimental results, allocate arrays etc
! Arguments (input):
!   * nHiggs= number of neutral Higgs in the model 
!     (see subroutine check_nH in input.f90 for more details)
!   * whichexpt_in= which combination of experimental results to use
!     (see subroutine check_whichexpt in input.f90 for more details)
!************************************************************
 use usefulbits, only : nH,debug,inputmethod,stat,theo,whichexpt,HiggsBounds_info
 use input, only : setup_input,check_nH,check_whichexpt
 use S95tables, only : setup_S95tables
 use theory_BRfunctions, only : setup_BRSM
 use channels, only : setup_channels
 use output, only : setup_output

#if defined(NAGf90Fortran)
 use F90_UNIX_IO, only : flush
#endif
 
 implicit none
 !--------------------------------------input
 integer,intent(in) :: nHiggs
 character(LEN=5),intent(in) :: whichexpt_in
 !-----------------------------------internal
 logical :: messages
 !-------------------------------------------

 stat=1
 debug=.False.
 inputmethod='subrout' !('datfile' or 'website' are also possible, but not here)
 
 nH=nHiggs
 call check_nH

 whichexpt=whichexpt_in
 call check_whichexpt 

 if(allocated(theo))then
  stop 'subroutine HiggsBounds_initialize has already been called once'
 endif 

 call HiggsBounds_info

 if(debug)write(*,*)'reading in Standard Model tables...'   ; call flush(6) 
 call setup_BRSM 
 		
 if(debug)write(*,*)'reading in S95tables...'               ; call flush(6)
 call setup_S95tables						

 if(debug)write(*,*)'doing other preliminary tasks...'      ; call flush(6)
 call setup_input

 if(debug)then
  open(20,file='debug_predratio.txt')
  open(21,file='debug_channels.txt')
 endif

 if(debug)write(*,*)'sorting out processes to be checked...'; call flush(6)
 call setup_channels	
	
 if(debug)write(*,*)'preparing output arrays...'            ; call flush(6)
 call setup_output
  
end subroutine initialize_HiggsBounds
!************************************************************	
subroutine run_HiggsBounds_effC(nHdummy,Mh,GammaTotal,             &   
     &          g2hjbb,g2hjtautau,g2hjWW,g2hjZZ,                   &
     &          g2hjgaga,g2hjgg,g2hjhiZ_nHbynH,                    &
     &          BR_hjhihi_nHbynH,                                  &
     &          HBresult,chan,                                     &
     &          obsratio, ncombined                                )
! This subroutine can be called by the user after subroutine initialize_HiggsBounds
! has been called.
! It takes theoretical predictions for a particular parameter point 
! in the model and calls subroutines which compare these predictions 
! to the experimental limits
! Arguments (input):
!   * nHdummy - checked to make sure it's the same as nH, but not otherwise used
!     (present in order to be consistent with the fortran77 version)
!   * Mh,GammaTotal,g2hjbb,g2hjtautau,g2hjWW,g2hjZZ,g2hjgaga,g2hjgg,g2hjhiZ_nHbynH,BR_hjhihi_nHbynH
!     are the required theoretical predictions (see manual for definitions)
! Arguments (output): 
!   * HBresult = 1 if point is unexcluded, 0 if excluded, -1 if parameter point is invalid
!   * chan = number of channel predicted to have the highest statistical sensitivity, as defined in Key.dat
!   * obsratio = ratio of the theoretical rate to the observed limit for this channel
!   * ncombined = number of Higgs combined in order to calculate this obsratio
!    (see manual for more precise definitions)
!************************************************************
 use usefulbits, only : theo,nH,g2,res,whichinput,debug,stat
 use input, only : test_input
 use channels, only : check_channels
 use theo_manip, only : complete_theo

#if defined(NAGf90Fortran)
 use F90_UNIX_IO, only : flush
#endif

 implicit none

 !----------------------------------------input
 integer,intent(in) :: nHdummy
 double precision,intent(in) :: Mh(nHdummy),GammaTotal(nHdummy),&   
     &          g2hjbb(nHdummy),g2hjtautau(nHdummy),            &
     &          g2hjWW(nHdummy),g2hjZZ(nHdummy),                &
     &          g2hjgaga(nHdummy),g2hjgg(nHdummy),              &
     &          g2hjhiZ_nHbynH(nHdummy,nHdummy),                &
     &          BR_hjhihi_nHbynH(nHdummy,nHdummy) 
 !----------------------------------------output
 integer,intent(out)::     HBresult,chan,ncombined
 double precision,intent(out) :: obsratio
 !--------------------------------------internal
 integer :: i,j,k,n
 !----------------------------------------------
 
 whichinput='effC'
 n=1  

 stat=stat+1
 if(.not.allocated(theo))then
  stop 'subroutine HiggsBounds_initialize must be called first'
 endif

 if(nHdummy.ne.nH)stop 'wrong input to subroutine run_HiggsBounds_effC'

 theo(n)%Mh              = Mh 
 theo(n)%GammaTot        = GammaTotal        
 g2(n)%hjbb              = g2hjbb
 g2(n)%hjtautau          = g2hjtautau
 g2(n)%hjWW              = g2hjWW
 g2(n)%hjZZ              = g2hjZZ       
 g2(n)%hjgaga            = g2hjgaga
 g2(n)%hjgg              = g2hjgg
 theo(n)%BR_hjhihi       = BR_hjhihi_nHbynH   

 k=0
 do j=1,nH
  do i=1,j
   k=k+1
   g2(n)%hjhiZ_in(k) = g2hjhiZ_nHbynH(j,i)
  enddo
 enddo

 !if(debug)call test_input(n)

 if(debug)write(*,*)'manipulating input...'                 ; call flush(6)
 call complete_theo

 if(debug)write(*,*)'compare each data point to the experimental bounds...' ; call flush(6)			
 call check_channels(theo(n),res(n))	 
	
 HBresult   = res(n)%allowed95(n)
 chan   = res(n)%chan(n)       
 obsratio    = res(n)%obsratio(n)
 ncombined   = res(n)%ncombined(n)	  

end subroutine run_HiggsBounds_effC
!************************************************************	
subroutine run_HiggsBounds_part(nHdummy,Mh,                  &
     &          CS_lep_hjZ_ratio,                            &
     &          CS_lep_hjhi_ratio_nHbynH,                    &
     &          CS_tev_gg_hj_ratio,CS_tev_bb_hj_ratio,       &
     &          CS_tev_bg_hjb_ratio,                         &
     &          CS_tev_ud_hjWp_ratio,CS_tev_cs_hjWp_ratio,   & 
     &          CS_tev_ud_hjWm_ratio,CS_tev_cs_hjWm_ratio,   & 
     &          CS_tev_dd_hjZ_ratio,CS_tev_uu_hjZ_ratio,     &
     &          CS_tev_ss_hjZ_ratio,CS_tev_cc_hjZ_ratio,     &
     &          CS_tev_bb_hjZ_ratio,                         &
     &          CS_tev_pp_vbf_ratio,                         &
     &          BR_hjbb,BR_hjtautau,                         &
     &          BR_hjWW,BR_hjgaga,                           & 
     &          BR_hjhihi_nHbynH,                            &
     &          HBresult,chan,                               &
     &          obsratio, ncombined                          )
! As for subroutine run_HiggsBounds_effC, but requiring different theoretical predictions
! (see manual for full description)
!************************************************************
 use usefulbits, only : theo,nH,res,tevR,whichinput,debug,stat
 use input, only : test_input
 use channels, only : check_channels
 use theo_manip, only : complete_theo

#if defined(NAGf90Fortran)
 use F90_UNIX_IO, only : flush
#endif

 implicit none
 !----------------------------------------input
 integer , intent(in) :: nHdummy
 double precision,intent(in) ::Mh(nHdummy),                                 &
     &          CS_lep_hjZ_ratio(nHdummy),                                  &
     &          CS_lep_hjhi_ratio_nHbynH(nHdummy,nHdummy),                  &
     &          CS_tev_gg_hj_ratio(nHdummy),CS_tev_bb_hj_ratio(nHdummy),    &
     &          CS_tev_bg_hjb_ratio(nHdummy),                               &
     &          CS_tev_ud_hjWp_ratio(nHdummy),CS_tev_cs_hjWp_ratio(nHdummy),& 
     &          CS_tev_ud_hjWm_ratio(nHdummy),CS_tev_cs_hjWm_ratio(nHdummy),& 
     &          CS_tev_dd_hjZ_ratio(nHdummy),CS_tev_uu_hjZ_ratio(nHdummy),  &
     &          CS_tev_ss_hjZ_ratio(nHdummy),CS_tev_cc_hjZ_ratio(nHdummy),  &
     &          CS_tev_bb_hjZ_ratio(nHdummy),                               &
     &          CS_tev_pp_vbf_ratio(nHdummy),                               &
     &          BR_hjbb(nHdummy),BR_hjtautau(nHdummy),                      &
     &          BR_hjWW(nHdummy),BR_hjgaga(nHdummy),                        &
     &          BR_hjhihi_nHbynH(nHdummy,nHdummy)
 !---------------------------------------output
 integer,intent(out) ::     HBresult,chan,ncombined
 double precision,intent(out) :: obsratio
 !---------------------------------------internal
 integer :: n
 !-----------------------------------------------

 if(nHdummy.ne.nH)stop 'wrong input to subroutine run_HiggsBounds_part'

 whichinput='part'
 n=1 
	
 stat=stat+1
 if(.not.allocated(theo))then
  stop 'subroutine HiggsBounds_initialize must be called first'
 endif

 theo(n)%Mh                = Mh        
 theo(n)%lep%XS_hjZ_ratio  = CS_lep_hjZ_ratio
 theo(n)%lep%XS_hjhi_ratio = CS_lep_hjhi_ratio_nHbynH 
 tevR(n)%gg_hj             = CS_tev_gg_hj_ratio 
 tevR(n)%qq_hj(5,:)        = CS_tev_bb_hj_ratio
 tevR(n)%bg_hjb            = CS_tev_bg_hjb_ratio                    
 tevR(n)%qq_hjWp(1,:)      = CS_tev_ud_hjWp_ratio
 tevR(n)%qq_hjWp(2,:)      = CS_tev_cs_hjWp_ratio         
 tevR(n)%qq_hjWm(1,:)      = CS_tev_ud_hjWm_ratio
 tevR(n)%qq_hjWm(2,:)      = CS_tev_cs_hjWm_ratio        
 tevR(n)%qq_hjZ(1,:)       = CS_tev_dd_hjZ_ratio
 tevR(n)%qq_hjZ(2,:)       = CS_tev_uu_hjZ_ratio           
 tevR(n)%qq_hjZ(3,:)       = CS_tev_ss_hjZ_ratio
 tevR(n)%qq_hjZ(4,:)       = CS_tev_cc_hjZ_ratio             
 tevR(n)%qq_hjZ(5,:)       = CS_tev_bb_hjZ_ratio                         
 theo(n)%tev%XS_vbf_ratio  = CS_tev_pp_vbf_ratio                         
 theo(n)%BR_hjbb           = BR_hjbb
 theo(n)%BR_hjtautau       = BR_hjtautau              
 theo(n)%BR_hjWW           = BR_hjWW
 theo(n)%BR_hjgaga         = BR_hjgaga                
 theo(n)%BR_hjhihi         = BR_hjhihi_nHbynH

 !if(debug)call test_input(n)

 if(debug)write(*,*)'manipulating input...'                 ; call flush(6)
 call complete_theo

 if(debug)write(*,*)'compare each data point to the experimental bounds...' ; call flush(6)			
 call check_channels(theo(n),res(n))	 
	
 HBresult   = res(n)%allowed95(n)
 chan   = res(n)%chan(n)       
 obsratio    = res(n)%obsratio(n)
 ncombined   = res(n)%ncombined(n)

end subroutine run_HiggsBounds_part
!************************************************************	
subroutine run_HiggsBounds_hadr(nHdummy,Mh,                  &
     &          CS_lep_hjZ_ratio,CS_lep_hjhi_ratio_nHbynH,   &
     &          CS_tev_pp_hj_ratio, CS_tev_pp_hjb_ratio,     &
     &          CS_tev_pp_hjW_ratio,CS_tev_pp_hjZ_ratio,     &
     &          CS_tev_pp_vbf_ratio,                         &
     &          BR_hjbb,BR_hjtautau,                         &
     &          BR_hjWW,BR_hjgaga,                           &
     &          BR_hjhihi_nHbynH,                            &
     &          HBresult,chan,                               &
     &          obsratio, ncombined                          )
! As for subroutine run_HiggsBounds_effC, but requiring different theoretical predictions
! (see manual for full description)
!************************************************************
 use usefulbits, only : theo,nH,res,whichinput,debug,stat
 use input, only : test_input
 use channels, only : check_channels
 use theo_manip, only : complete_theo

#if defined(NAGf90Fortran)
 use F90_UNIX_IO, only : flush
#endif

 implicit none
 !----------------------------------------input
 integer,intent(in) :: nHdummy
 double precision,intent(in) :: Mh(nHdummy),              &
     &          CS_lep_hjZ_ratio(nHdummy),                &
     &          CS_lep_hjhi_ratio_nHbynH(nHdummy,nHdummy),&
     &          CS_tev_pp_hj_ratio(nHdummy),              &
     &          CS_tev_pp_hjb_ratio(nHdummy),             &
     &          CS_tev_pp_hjW_ratio(nHdummy),             &
     &          CS_tev_pp_hjZ_ratio(nHdummy),             &
     &          CS_tev_pp_vbf_ratio(nHdummy),             &
     &          BR_hjbb(nHdummy),BR_hjtautau(nHdummy),    &
     &          BR_hjWW(nHdummy),BR_hjgaga(nHdummy),      &
     &          BR_hjhihi_nHbynH(nHdummy,nHdummy)
 !---------------------------------------output
 integer,intent(out) ::     HBresult,chan,ncombined
 double precision,intent(out) :: obsratio
 !-------------------------------------internal
 integer :: n
 !---------------------------------------------

 if(nHdummy.ne.nH)stop 'wrong input to subroutine run_HiggsBounds_hadr'
                    
 whichinput='hadr'
 n=1 

 stat=stat+1
 if(.not.allocated(theo))then
  stop 'subroutine HiggsBounds_initialize must be called first'
 endif

 theo(n)%Mh                = Mh              
 theo(n)%lep%XS_hjZ_ratio  = CS_lep_hjZ_ratio
 theo(n)%lep%XS_hjhi_ratio = CS_lep_hjhi_ratio_nHbynH 
 theo(n)%tev%XS_hj_ratio   = CS_tev_pp_hj_ratio
 theo(n)%tev%XS_hjb_ratio  = CS_tev_pp_hjb_ratio   
 theo(n)%tev%XS_hjW_ratio  = CS_tev_pp_hjW_ratio
 theo(n)%tev%XS_hjZ_ratio  = CS_tev_pp_hjZ_ratio    
 theo(n)%tev%XS_vbf_ratio  = CS_tev_pp_vbf_ratio      
 theo(n)%BR_hjbb           = BR_hjbb
 theo(n)%BR_hjtautau       = BR_hjtautau                 
 theo(n)%BR_hjWW           = BR_hjWW
 theo(n)%BR_hjgaga         = BR_hjgaga                   
 theo(n)%BR_hjhihi         = BR_hjhihi_nHbynH
    
 !if(debug)call test_input(n)

 if(debug)write(*,*)'manipulating input...'                 ; call flush(6)
 call complete_theo

 if(debug)write(*,*)'compare each data point to the experimental bounds...' ; call flush(6)			
 call check_channels(theo(n),res(n))	 
	
 HBresult   = res(n)%allowed95(n)
 chan   = res(n)%chan(n)       
 obsratio    = res(n)%obsratio(n)
 ncombined   = res(n)%ncombined(n)

end subroutine run_HiggsBounds_hadr
!************************************************************
subroutine finish_HiggsBounds
! This subroutine needs to be called right at the end, to close files
! and deallocate arrays
!************************************************************
 use usefulbits, only : deallocate_usefulbits,debug,stat,theo,debug
 use S95tables, only : deallocate_S95tables
 use theory_BRfunctions, only : deallocate_BRSM

#if defined(NAGf90Fortran)
 use F90_UNIX_IO, only : flush
#endif
	
 if(debug)then
  close(20)
  close(21)
 endif

 if(.not.allocated(theo))then
  stop 'subroutines HiggsBounds_initialize,run_HiggsBounds_*  should be called first'
 elseif(stat.eq.1)then 
  write(*,*) 'Warning: subroutine run_HiggsBounds_* has not been called' 
 endif

 if(debug)write(*,*)'finishing off...'                      ; call flush(6)
 call deallocate_BRSM
 call deallocate_S95tables
 call deallocate_usefulbits
 if(debug)write(*,*)'finished'                              ; call flush(6)
 
 stat=0
end subroutine finish_HiggsBounds
!************************************************************

